/*
 *	Copyright (C) 2007, 2008, 2009, 2010. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVDEFINES_H
#define QVDEFINES_H

// Include a small set of Qt classes, which will be declared as basic types for the metatypes.
#include <QMetaType>
#include <QList>
#include <QVector>
#include <QPoint>
#include <QPointF>
#include <QRect>
#include <QPair>
#include <limits>

typedef unsigned char 	uChar;		// Ipp8u	1 bytes
typedef unsigned short 	uShort;		// Ipp16u	2 "
typedef unsigned int 	uInt;		// Ipp32u	4 "
typedef signed char 	sChar;		// Ipp8s	1 ...
typedef signed short 	sShort;		// Ipp16s	2
typedef signed int 	sInt;		// Ipp32s	4
//typedef __INT64 	sLong;		// Ipp64s	8 (not an estandar type)
//typedef __UINT64 	uLong;		// Ipp64u	8 (not an estandar type)
typedef float 		sFloat;		// Ipp32f	4
typedef double 		sDouble;	// Ipp64f	8

// Other type definitions
typedef QPair<QPointF, QPointF> QPointFMatching;

// Qt metatypes definitions: QVision defined types must be metatyped in their respective header files.
// Qt's and C++'s metatypes must be listed below.
Q_DECLARE_METATYPE(sChar);
Q_DECLARE_METATYPE(QVector<sFloat>);
Q_DECLARE_METATYPE(QVector<sDouble>);
Q_DECLARE_METATYPE(QList<sInt>);
Q_DECLARE_METATYPE(QList<sFloat>);
Q_DECLARE_METATYPE(QList<sDouble>);
Q_DECLARE_METATYPE(QList<QPoint>);
Q_DECLARE_METATYPE(QList<QPointF>);
Q_DECLARE_METATYPE(QList<QRect>);
Q_DECLARE_METATYPE(QList< QList< QPoint > >);
Q_DECLARE_METATYPE(QList< QList< QPointF > >);

Q_DECLARE_METATYPE(QPointFMatching);
Q_DECLARE_METATYPE(QList<QPointFMatching>);

#include <QColor>
const QColor qvColors[] = {
		Qt::red,
		Qt::green,
		Qt::blue,
		Qt::cyan,
		Qt::magenta,
		Qt::darkRed,
		Qt::darkGreen,
		Qt::darkBlue,
		Qt::darkCyan,
		Qt::darkMagenta
		};

std::ostream& operator << (std::ostream &os, const QPointF &point);
std::ostream& operator << (std::ostream &os, const QPoint &point);
std::ostream& operator << (std::ostream &os, const QPointFMatching &matching);

uint qHash(const double &value);
uint qHash(const QPointF &point);
uint qHash(const QPoint &point);

#ifdef QVIPP
#include <ipp.h>
Q_DECLARE_METATYPE(IppiMaskSize);
Q_DECLARE_METATYPE(IppCmpOp);
Q_DECLARE_METATYPE(IppRoundMode);
#endif

#endif


